//******************
// DPAPI.H
//
// Header file for DPAPI.C
// Includes inline ASM for a TASM-free DIGPAK api
// Please refer to the DIGPAK documentation for further notes on
// the various functions
//
// Copyright (c) 1996 Manuel Suarez
// abalamahalamatandra@pobox.com
//
// NOTICE:  My files, DPAPI*, DPMIX* & DPAM*, are FREE SOFTWARE, you may use or
// modify them for any purpose, provided I get credit for my work
// (such as it is)
//
// DIGPAK is a product of The Audio Solution, free for non-commercial use
// with a $500 licensing fee if you sell 500 copies of a commercial work
//
// The inline ASM code in this file is a pretty straight forward conversion
// of the ASM code in the DIGPAK developer's kit
//******************

#ifndef DPAPI_H
#define DPAPI_H

//******************
// defines
//******************

// buffer size to allocate with DpInstall
// It has been observed that values >=32768 do not work
// and that the length must be a multiple of 16
// otherwise clicking will occur, thus (32768-16) is the maximum buffer
// size.  By default, this is allocated, and up to half this value
// can be passed to DpOn
#define DP_DMA_BUFSIZE (32768-16)
#define DP_DMA_HALFBUF (DP_DMA_BUFSIZE/2)

// capabilities
#define DP_BACK      (1) // background playing supported
#define DP_FORMAT    (2) // data must be reformatted
#define DP_FIXED     (4) // frequency is fixed, driver will downsample
#define DP_TIMER     (8) // driver uses the timer interrupt
#define DP_SHARE    (16) // driver supports sharing the timer
#define DP_PEND     (32) // driver supports pending sounds
#define DP_PAN      (64) // driver supports stereo panning
#define DP_8S      (128) // driver supports 8 bit stereo
#define DP_RECORD  (256) // driver suports recording
#define DP_DMA     (512) // driver supports DMA backfilling
#define DP_16M    (1024) // driver supports 16 bit mono
#define DP_16S    (2048) // driver supports 16 bit stereo

// play modes
#define DP_MODE_8M   (0)  // 8 bit mono
#define DP_MODE_8S   (1)  // 8 bit stereo
#define DP_MODE_16M  (2)  // 16 bit mono
#define DP_MODE_16S  (3)  // 16 bit stereo

//******************
// DIGPAK buffer structure
//******************
typedef struct _DpBuffer{
  unsigned short BufOffset;
  unsigned short BufSegment;
  unsigned short Length;
  unsigned short StatusOffset;
  unsigned short StatusSegment;
  unsigned short Frequency;
} DpBuffer;

//******************
// real mode call structure
//******************
typedef struct _DpRmi{
  int edi,esi,ebp,res1,ebx,edx,ecx,eax;
  short flags,es,ds,fs,gs,ip,cs,sp,ss;
} DpRmi;

//******************
// raw sample structure
//******************
typedef struct _DpSample{
  int Mode;       // Mode of the sample (8m, 8s, as above)
  int Frequency;  // Frequency recorded at
  int Size;       // size in bytes
  void *Buffer;   // address of the buffer
} DpSample;

//******************
// .dps file header
//******************
#define DP_DPS_VERSION (1)
typedef struct _DpDpsHeader{
  int Tag;        // "FSPD" in file, test for Tag='DPSF'  (Intel scrambled)
  int Version;    // DP_DPS_VERSION
  int Mode;       // Mode of the sample
  int Frequency;  // Frequency of the sample (samples per second)
  int Size;       // Size of the sample in bytes
} DpDpsHeader;

//******************
// .wav format chunk
// info from David Welch's
// readme & distribution of riffmci.txt
//******************
typedef struct _WaveFormatChunk{
  short Tag;
  short Channels;
  long SampleRate;
  long ByteRate;
  short Block;
  short Bits;
} WaveFormatChunk;

// various id's intel scrambled for longword comparison
#define WAV_RIFFID (0x46464952)
#define WAV_WAVEID (0x45564157)
#define WAV_FMTID  (0x20746d66)
#define WAV_DATAID (0x61746164)

//******************
// DPAPI.C prototypes & globals
//******************
extern DpBuffer *DpBuf;
extern volatile unsigned short *DpFlag;
extern volatile unsigned short *DpSem;
extern volatile unsigned short *DpPending;
extern void *DpDMAbuf;
extern int DpCap;
extern int DpMaxMode;
extern char *DpModeName[4];
extern char *DpTypeName[4];

void *DpAllocateLow(int size,int *selector);
int DpFreeLow(int selector);
int DpCheck(void);
int DpLoad(int flag);
void DpPrintId(void);
void DpRunLoud(void);
void DpRunQuiet(void);
void DpPause(int timer);
int DpInstall(void);
void DpRemove(void);
int DpOn(int mode,int frequency,int bufsize);
void DpOff(void);
void DpClearDMAbuf(int mode);
DpSample *DpLoadSample(char *filename, int *type);
int DpSaveSample(char *filename,DpSample *ds,int type);
void DpFreeSample(DpSample *sample);
DpSample *DpLoadRaw(char *filename);
int DpSaveRaw(char *filename,DpSample *ds);
DpSample *DpLoadDps(char *filename);
int DpSaveDps(char *filename,DpSample *ds);
DpSample *DpLoadWav(char *filename);
int DpSaveWav(char *filename,DpSample *ds);
int DpDmacKludge(void);

//******************
//  inline asm versions of
//  selected DIGPAK api functions
//
// November 6, 1996 : found DpDigPlay modifies eax,ebx,ecx and edx
//                    aren't debugger's wonderful?  Now runs w/o optimizations
// November 3, 1996 : After much frustration, tracked down an "undocumented
//                    feature" of DpVerifyDMA: the modification of edx.
//                    This coupled with optimizations was causing the buffer
//                    pointer to be corrupted.  There may be similar
//                    problems with other DIGPAK calls.
// November 2, 1996 : consistent "Dp*" naming
// March 9, 1996 : incorporated into header file
// January, 1996 : initial version
//******************

// api inline asm functions
extern void DpDigPlay(void *);
extern void DpSetDPMIMode(void);
extern int DpReportVersionNumber(void);
extern int DpAudioCapabilities(void);
extern void DpStopSound(void);
extern int DpPostPending(void *);
extern int DpSetPlayMode(int);
extern unsigned short *DpReportSemaphoreAddress(void);
extern unsigned short *DpReportPendingAddress(void);
extern int DpSetRecordMode(int);
extern int DpSetBackFillMode(int);
extern int DpReportDmac(void);
extern int DpVerifyDMA(void *,int);
extern void DpWaitSound(void);

//******************
// DpDigPlay(DpBuffer structure)
// start the sound playing
//******************
#pragma aux DpDigPlay= \
  "mov eax,0x688" \
  "int 0x66" \
  parm [esi] \
  modify [eax ebx ecx edx];

//******************
// DpSetDPMIMode
// set calling convention for protected mode
//******************
#pragma aux DpSetDPMIMode=\
  "mov edx,0x1" \
  "mov eax,0x6a0" \
  "int 0x66" \
  modify [eax edx];

//******************
// DpReportVersionNumber
// returns the driver's version number
//******************
#pragma aux DpReportVersionNumber= \
  "xor ebx,ebx" \
  "mov eax,0x689" \
  "int 0x66" \
  "mov eax,ebx" \
  modify [ebx edx] \
  value [eax];

//******************
// DpAudioCapabilities
// returns the driver's audio capabilities
//******************
#pragma aux DpAudioCapabilities= \
  "mov eax,0x68c" \
  "int 0x66" \
  modify [ebx edx] \
  value [eax];

//******************
// DpStopSound
// stops sound playback
//******************
#pragma aux DpStopSound= \
  "mov eax,0x68f" \
  "int 0x66" \
  modify [eax];

//******************
// DpPostPending(DpBuffer structure)
// post a sound to play when the current one finishes
// returns 0 for playing, 1 for pending, 2 for something already pending
//******************
#pragma aux DpPostPending= \
  "mov eax,0x695" \
  "int 0x66" \
  parm [esi] \
  value [eax];

//******************
// DpSetPlayMode(mode)
// 0 8 bit mono
// 1 8 bit stereo
// 2 16 bit mono
// 3 16 bit stereo
// returns 1 for success, 0 for failure
//******************
#pragma aux DpSetPlayMode= \
  "mov eax,0x698" \
  "int 0x66" \
  parm [edx] \
  value [eax];

//******************
// DpReportSemaphoreAddress
// returns the address of the DIGPAK semaphore
// test semaphore for 0 before calling DIGPAK functions
//******************
#pragma aux DpReportSemaphoreAddress= \
  "xor edx,edx" \
  "xor ebx,ebx" \
  "mov eax,0x699" \
  "int 0x66" \
  "mov eax,edx" \
  "shl eax,4" \
  "add eax,ebx" \
  modify [ebx edx] \
  value [eax];

//******************
// DpReportPendingAddress
// returns the address of the DIGPAK pending flag
// flag is 0 for ok to pend
//******************
#pragma aux DpReportPendingAddress= \
  "xor edx,edx" \
  "xor ebx,ebx" \
  "mov eax,0x699" \
  "int 0x66" \
  "shl edx,4" \
  "add eax,edx" \
  modify [ebx edx] \
  value [eax];

//******************
// DpSetRecordMode(mode)
// 0 record instead of play
// 1 play
// returns 1 for success, 0 for failure
//******************
#pragma aux DpSetRecordMode= \
  "mov eax,0x69a" \
  "int 0x66" \
  parm [edx] \
  value [eax];

//******************
// DpSetBackFillMode(mode)
// 0 backfill off (no looping)
// 1 backfill on  (looping)
// returns 1 for success, 0 for failure
//******************
#pragma aux DpSetBackFillMode= \
  "mov eax,0x69c" \
  "int 0x66" \
  parm [edx] \
  value [eax];

//******************
// DpReportDmac
// returns position of DMA counter
// (counts down, SB16 reports only 2 values, but they are enough)
//******************
#pragma aux DpReportDmac= \
  "mov eax,0x69d" \
  "int 0x66" \
  value [eax];

//******************
// DpVerifyDMA(buffer,length)
// returns 1 if block is OK
// returns 0 if block is invalid due to crossing 64k boundary
// allocate a double size buffer and use second half
// if first half reports invalid
//******************
#pragma aux DpVerifyDMA= \
  "mov eax,0x69e" \
  "int 0x66" \
  parm [ebx][ecx] \
  modify [edx] \
  value [eax];

//******************
// DpWaitSound
// waits until current sound finished
// NOTICE: This will hang on looping (DMA autoinit) play!!!
//******************
#pragma aux DpWaitSound= \
  "notready: mov eax,0x689" \
  "int 0x66" \
  "or ax,ax" \
  "jnz notready" \
  modify [eax ebx edx]; 

#endif
