//******************
// TIMER.C
// fast timer w/18.2 hz system callback & variable user callback
//
// Copyright (c) 1996 Manuel Suarez
// abalamahalamatandra@pobox.com
//
// USAGE:  TimerInstall(value) to install
//         TimerCallback(value,callback) to install callback function
//         TimerRemove() to remove
//         The global variable Timer is incremented at each interrupt
//
//         To remove the callback before TimerRemove, use TimerCallback(0,NULL)
//
//         value is TIMER_FREQ/frequency  (1193180)
//         install range is 0-65535, with 0 representing a divisor of 65536
//         callback value can be >65535, and must be >= install value
//         see TIMER.H for some pre-defined values
//
// November 6, 1996 : cleaned it up a little, made callback near
//                    TimerRemove() checks first to see if installed
// November 4, 1996 : added a no frills callback function
// October 28, 1996 : converted to inline asm
//******************

#include <dos.h>  // get/set interrupt

//******************
// global data
//******************
volatile int Timer;
static int TimerEt,TimerEtAdj;
static int TimerCbEt,TimerCbReset;
static void (*TimerCbFn)(void);
static void (__interrupt __far *OldInt08)();

//******************
// prototypes for inline asm
//******************
extern void TimerSetHardware(int timerval);
extern void InlineInt08Isr(void);
extern void __GETDS(void);
#pragma aux __GETDS "*";

//******************
// TimerSetHardware
// sets hardware clock to specified value
//******************
#pragma aux TimerSetHardware= \
"  push eax           " /* save eax                     */  \
"  cli                " /* no interruptions             */  \
"  push eax           " \
"  mov al,36h         " /* set timer 0                  */  \
"  out 43h,al         " \
"  pop eax            " \
"  out 40h,al         " /* set low byte                 */  \
"  mov al,ah          " /* set high byte               */  \
"  out 40h,al         " \
"  sti                " /* int's ok now                 */  \
"  pop eax            " /* restore eax                  */  \
parm [eax];

//******************
// Inline ASM ISR
// increments Timer and calculates when to call system ISR & callback
// This really oughta just be written in C, but what the heck, it works
//******************
void InlineInt08Isr1(void);
void InlineInt08Isr2(void);
#pragma aux InlineInt08Isr1= \
"  push ds            " /* save ds                      */  \
"  call __GETDS       " /* borrow Watcom's get DS routine*/ \
"  push eax           " /* save eax                     */  \
"                     "\
"  inc Timer          " /* increment the timer          */  \
"                     "\
"  cmp TimerCbFn,0    " /* is there a callback?         */  \
"  je NoCallback      " /* nope                         */  \
"                     "\
"  mov eax,TimerCbEt  " /* get the callback error term  */  \
"  sub eax,TimerEtAdj " /* adjust it                    */  \
"  jg NoCallback      " /* not time for the callback yet*/  \
"                      "\
"  add eax,TimerCbReset"/* reset the error term         */  \
"  mov TimerCbEt,eax  " /* save callback error term     */  \
"                     "\
"  call dword ptr TimerCbFn" /* run the callback        */  \
"                     "\
"  jmp CallbackDone   "\
"                     "\
"NoCallback:          "\
"  mov TimerCbEt,eax  " /* save callback error term     */  \
"CallbackDone:        ";

#pragma aux InlineInt08Isr2= \
"  mov eax,TimerEt    " /* get the timer error term     */  \
"  sub eax,TimerEtAdj " /* adjust it                    */  \
"  jle _I8ETtripped   " /* <=0, time to run old interrupt */ \
"                     "\
"  mov TimerET,eax    " /* otherwise, save the error term */ \
"  mov al,20h         " /* and clear the interrupt      */  \
"  out 20h,al         " \
"  pop eax            " /* restore eax                  */  \
"  pop ds             " /* restore ds                   */  \
"  iretd              " /* return                       */  \
"                     "\
"_I8ETtripped:        " \
"  add eax,65536      " /* reset the error term         */  \
"  mov TimerET,eax    " /* save the error term          */  \
"  pop eax            " /* restore eax                  */  \
"  pop ds             " /* restore ds                   */  \
"  jmp pword ptr cs:OldInt08 "; /* and let the old interrupt run */ \

//******************
// The new Int 0x08 ISR
// (just somewhere to put the inline ISR)
// NOTE:  This can't be type "__interrupt" or the compiler will
// save everything at the beginning and none of it will be
// restored
//******************
#ifndef __SW_S
#pragma off (check_stack)
#endif
#pragma aux Int08Isr aborts;
void Int08Isr(void){
  InlineInt08Isr1();
  InlineInt08Isr2();
} // end TimerISR
#ifndef __SW_S
#pragma on (check_stack)
#endif

//******************
// TimerInstall(value)
// sets hardware timer frequency & sets up new ISR
// range is 0-65535  (0=65536)
//******************
void TimerInstall(unsigned short timerval){
  Timer=0;
  TimerEt=65536;
  TimerEtAdj=(timerval ? timerval : 65536);
  TimerCbFn=0;
  TimerSetHardware(timerval);
  OldInt08=_dos_getvect(0x08);
  _dos_setvect(0x08,(void *)Int08Isr);
} // end TimerInstall

//******************
// TimerCallback(value,function)
// installs user callback function at given timer value
// This value may be >65535, if a slow timer is desired
// The value MUST be >= the base timer value (frequency<=base)
// To remove the callback just set a function of NULL
//
// DS is already set & EAX is saved, stack is whatever DPMI host provided
// CALLBACK IS RESPONSIBLE FOR SAVING ALL OTHER REGISTERS USED!
// Conveniently, Watcom C does, but be sure to WDISASM whatever .obj module
// the callback is in and see what it's really doing
//
// Remember, it is the base timer frequency, not the callback frequency,
// which governs how much time you can get away with within the callback
// and, of course, the rest of the program needs to run between interrupts!
//
// The callback and EVERYTHING called from the callback should be
// surrounded by this, else it will choke if compiled w/o optimizations
//
// #ifndef __SW_S
// #pragma off (check_stack)
// #endif
// ...
// #ifndef __SW_S
// #pragma on (check_stack)
// #endif
//******************
void TimerCallback(int timerval,void (*function)(void)){
  if(!timerval) timerval=65536;
  if(timerval<TimerEtAdj) timerval=TimerEtAdj; // just in case
  TimerCbEt=timerval;
  TimerCbReset=timerval;
  TimerCbFn=function;
} // end TimerCallback

//******************
// TimerRemove()
// restores the hardware timer frequency and the old ISR
//******************
void TimerRemove(){
  if(_dos_getvect(0x08)==Int08Isr) _dos_setvect(0x08,OldInt08);
} // end TimerRemove



